var events = require('events');

var settings = require('../../settings.json');
var Promise = require("bluebird");

exports.required = ['getLatest', 'delay'];
exports.optional = ['timeout'];

exports.func = function (args) {
  var latest = args.getLatest;
  var delay = args.delay;
  delay = typeof delay === 'string' || delay instanceof String ? settings.event[delay] : delay;
  var retries = 0;
  var max = settings.event.maxRetries;
  var timeout = args.timeout || settings.event.timeout;
  var stop = false;
  var current;
  var evt = new events.EventEmitter();
  var run;

  run = function (value) {
    if (stop) {
      return;
    }
    var promise = latest(value, evt);
    if (timeout > 0) {
      promise = promise.timeout(timeout);
    }
    return promise.then(function (response) {
      if (stop) {
        return;
      }
      if (value === -1) {
        current = response.latest;
      }
      var data = response.data;
      if (data.length > 0 && (value !== -1 || current === -2)) {
        current = response.latest;
        for (let i = 0; i < data.length; i++) {
          evt.emit('data', data[i]);
        }
      }
      retries = 0;
      if (response.repeat) {
        run(current);
      } else {
        setTimeout(run, delay, current);
      }
      return response;
    })
    .catch(function (err) {
      if (stop) {
        return;
      }
      retries++;
      if (retries > max) {
        evt.emit('close', "Max retries reached! " + err);
      } else {
        setTimeout(run, delay, current);
      }
    });
  };
  function start() {
    run(-1)
    .then(function (response) {
      if (stop) {
        return;
      }
      evt.emit('connect', response.latest);
    })
    .catch(function (err) {
      retries++;
      if (retries > max) {
        evt.emit('close', 'Initialization failed: ' + err.message);
      } else {
        setTimeout(start, delay);
      }
    });
  }
  start();
  evt.on('close', function (err) {
    stop = true;
    if (err) {
      evt.emit('error', err);
    }
  });
  return evt;
};
